package com.example.aped.communication;

import android.content.Context;
import android.util.Log;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.Volley;
import com.example.aped.utils.ExternalStorageHandler;

import org.json.JSONObject;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * Class that handles communication with the raspberry pi.
 */
public class Communicator {

    /** Ipv4 address of the raspberry. **/
    private final String address;
    /** API port of the raspberry. **/
    private final int port;
    /** Request queue used for asynchronous requests. **/
    private final RequestQueue requestQueue;
    /**The context of the application.**/
    private final Context context;

    /**
     * Constructor of the Communicator.
     * @param pAddress the ipv4 address of the raspberry
     * @param pPort the port of the raspberry
     * @param pContext the context of the application
     */
    public Communicator(final String pAddress,
                        final int pPort,
                        final Context pContext) {
        this.address = pAddress;
        this.port = pPort;
        requestQueue = Volley.newRequestQueue(pContext);
        this.context = pContext;
    }

    /**
     * Reads the device state from the raspberry.
     * @param deviceName the name of the device
     * @param responseListener the {@link Response.Listener} for defining
     *                         actions to be performed on response
     */
    public void read(final String deviceName,
                     final Response.Listener<JSONObject> responseListener) {
        String url = "http://" + address
                + ":" + port
                + "/device/" + deviceName + "/";
        JsonObjectRequest request = new JsonObjectRequest(
                Request.Method.GET,
                url,
                null,
                responseListener,
                error -> Log.e("Communicator",
                        "Error during READ: " + error.getMessage())
        );
        requestQueue.add(request);
    }

    /**
     * Writes an api call to the endpoint of a device.
     * @param deviceName the name of the device
     * @param message the JSON message
     * @param responseListener the {@link Response.Listener} for defining
     *                         actions to be performed on response
     */
    public void write(final String deviceName,
                      final JSONObject message,
                      final Response.Listener<JSONObject> responseListener) {
        String url = "http://" + address
                + ":" + port
                + "/device/" + deviceName + "/";
        JsonObjectRequest request = new JsonObjectRequest(
                Request.Method.POST,
                url,
                message,
                responseListener,
                error -> Log.e("Communicator",
                        "Error during WRITE: " + error.getMessage())
        );
        requestQueue.add(request);
    }

    /**
     * Uploads the config.xml file to the raspberry.
     */
    public void uploadXML() throws FileNotFoundException {
        File xmlFile = new File(ExternalStorageHandler
                .getExternalPrivateStorageDir(context), "config.xml");
        String url = "http://" + address + ":" + port + "/XML/";
        FileOutputVolleyRequest request = new FileOutputVolleyRequest(
                Request.Method.POST,
                url,
                xmlFile,
                response -> System.out.println(response.toString()),
                error -> Log.e("Communicator",
                        "Error during XML UPLOAD" + error.getMessage())
        );
        requestQueue.add(request);
    }

    /**
     * Downloads the config.xml file from the raspberry.
     */
    public void downloadXML() {
        String url = "http://" + address + ":" + port + "/XML/";
        InputStreamVolleyRequest request = new InputStreamVolleyRequest(
                Request.Method.GET,
                url,
                response -> {
                    try {
                        if (response != null) {
                            String xmlPath = ExternalStorageHandler
                                    .getExternalPrivateStorageDir(context);
                            File xmlFile = new File(xmlPath, "config.xml");
                            /*create the default XML config by using the
                            default.xml from the assets folder*/
                            if (xmlFile.exists()) {
                                xmlFile.delete();
                            }
                            FileOutputStream fileOutputStream =
                                    new FileOutputStream(xmlFile);
                            fileOutputStream.write(response);
                            fileOutputStream.close();
                        }
                    } catch (IOException e) {
                        Log.e("Communicator",
                                "Error while reading the XML DOWNLOAD "
                                        + "from the raspberry: "
                                        + e.getMessage());
                    }
                },
                error -> Log.e("Communicator",
                        "Error during XML DOWNLOAD: " + error.getMessage())
        );
        requestQueue.add(request);
    }
}
